import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

abstract class RxTabPage<Tab> extends StatelessWidget {
  Widget backdrop() => Container();

  RxModel<List<Tab>> get tabs;

  Widget innerPage(Tab tab);

  Widget appBar(PreferredSizeWidget tabBar);

  Widget tabBuilder(Tab tab, bool isSelected);

  Rx<int> get page => 0.obs;

  RxTabBar _tabBar(
    Rx<int> page,
    List<Tab> tabs,
  ) =>
      RxTabBar<Tab>(
        page: page,
        source: tabs,
        builder: tabBuilder,
      );

  @override
  Widget build(BuildContext context) => Scaffold(
        body: tabs.toWidget(
          (tabs) => DefaultTabController(
            length: tabs.value.length,
            child: PageDetector(
              page: page,
              child: (page) => NestedScrollView(
                headerSliverBuilder: (_, __) => [
                  appBar(_tabBar(page, tabs.value)),
                ],
                body: Stack(
                  children: [
                    Positioned.fill(child: backdrop()),
                    TabBarView(
                      children: [
                        ...tabs.value.map(innerPage),
                      ],
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      );
}
